import API from "@/services/imapAPI";
import { createStore } from "vuex";
import Room from "./models/Room";
import Account from "./models/Account";

const store = createStore({
    state() {
        return {
            rooms: [],
            messages: [],
            accounts: [new Account(0, "ALL")],
            activeAccount: 0,
            activeRoom: 0,
        };
    },
    mutations: {
        setactiveAccount(state, payload) {
            state.activeAccount = payload;
            const account = state.accounts.find((account) => account.id == payload);
            store.dispatch("fetchRooms", { accountId: payload, account: account });
        },
        setActiveRoom(state, payload) {
            state.activeRoom = payload;
            // todo call actions
            // fetch messages for this room if not already fetched
            const room = state.rooms.find((room) => room.id == payload);
            if (!room || room?.fetched == false) {
                console.log("add messages");
                API.getMessages(payload)
                    .then((res) => {
                        // todo add if not exist
                        room.fetched = true;
                        res.data.forEach((msg) => {
                            state.messages.push(msg);
                        });
                    })
                    .catch((err) => {
                        console.log(err);
                    });
            }
        },
        addAccounts(state, payload) {
            payload.forEach((account) => {
                state.accounts.push(new Account(account.id, account.email));
            });
        },
        addRooms(state, payload) {
            // todo add if not exist
            payload.rooms.forEach((room) => {
                state.rooms.push(new Room(room.id, room.user, room.userId, room.roomName, room.mailboxId));
            });
        },
        addMessages(state, payload) {
            const room = state.rooms.find((room) => room.id == payload.roomId);
            payload.messages.forEach((message) => {
                room.messages.push(message);
            });
            room.messagesFetched = true;
        },
    },
    getters: {
        rooms: (state) => () => {
            if (state.activeAccount === 0) return state.rooms;
            return state.rooms.filter((room) => room.mailboxId == state.activeAccount);
        },
        messages: (state) => (roomId) => {
            const room = state.rooms.find((room) => room.id == roomId);
            if (!room.messagesFetched) {
                store.dispatch("fetchMessages", { roomId: room.id });
            }
            return room.messages;
        },
    },
    actions: {
        fetchAccounts: async (context) => {
            API.getAccounts()
                .then((res) => {
                    context.commit("addAccounts", res.data);
                })
                .catch((err) => {
                    console.log(err);
                });
        },
        fetchRooms: async (context, data) => {
            if (data.account?.fetched == false) {
                API.getRooms(data.accountId)
                    .then((res) => {
                        data.account.fetched = true;
                        context.commit("addRooms", { rooms: res.data });
                    })
                    .catch((err) => {
                        console.log(err);
                    });
            }
        },
        fetchMessages: async (context, data) => {
            console.log(data)
            API.getMessages(data.roomId)
                .then((res) => {
                    console.log(res.data);
                    context.commit("addMessages", { messages: res.data, roomId: data.roomId });
                })
                .catch((err) => {
                    console.log(err);
                });
        },
    },
});

export default store;
